package api

import (
	"encoding/json"
	"net/http"

	"github.com/google/uuid"
	"github.com/gorilla/mux"
)

// Item data structure SUB
type Item struct {
	ID   uuid.UUID `json:"id"`
	Name string    `json:"name"`
}

// Server data structure MAIN
type Server struct {
	*mux.Router   // Merge mux router comportement with Server
	ShoppingItems []Item
}

// Server Constructor
func NewServer() *Server {
	server := &Server{
		Router:        mux.NewRouter(),
		ShoppingItems: []Item{},
	}
	server.InitRoutes()
	return server
}

// Init mux router
func (server *Server) InitRoutes() {
	server.HandleFunc("/shopping-items", server.ListShoppingItems()).Methods("GET")
	server.HandleFunc("/shopping-items", server.CreateShoppingItem()).Methods("POST")
	server.HandleFunc("/shopping-items/{id}", server.RemoveShoppingItem()).Methods("DELETE")
}

// Handler create item
func (server *Server) CreateShoppingItem() http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		var item Item
		if err := json.NewDecoder(r.Body).Decode(&item); err != nil {
			http.Error(w, err.Error(), http.StatusBadRequest)
			return
		}

		item.ID = uuid.New()
		server.ShoppingItems = append(server.ShoppingItems, item)

		w.Header().Set("Content-Type", "application/json")
		if err := json.NewEncoder(w).Encode(item); err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
	}
}

// Handler retrieve items
func (server *Server) ListShoppingItems() http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		w.Header().Set("Content-Type", "application/json")
		if err := json.NewEncoder(w).Encode(server.ShoppingItems); err != nil {
			http.Error(w, err.Error(), http.StatusInternalServerError)
			return
		}
	}
}

// Hndler remove item
func (server *Server) RemoveShoppingItem() http.HandlerFunc {
	return func(w http.ResponseWriter, r *http.Request) {
		rawID := mux.Vars(r)["id"]

		itemID, err := uuid.Parse(rawID)
		if err != nil {
			http.Error(w, err.Error(), http.StatusBadRequest)
			return
		}

		for i, item := range server.ShoppingItems {
			if item.ID == itemID {
				server.ShoppingItems = append(server.ShoppingItems[:i], server.ShoppingItems[i+1:]...)
				break
			}
		}
	}
}
